
#include "Arduino.h"

#define ENABLE 6
#define IN1 5 // OC3A (timer 3, Output compare A), PE3 (port E sortie 3)
#define IN2 2 // OC3B (timer 3, Output compare A), PE4 (port 3 sortie 4)
 
char clockBits;
volatile uint32_t count;
int32_t max_count;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
uint32_t icr;


void pulse_train(uint32_t period, int32_t nombre, float rapport) { 
  TCCR3A = 0;
  TCCR3A |= (1 << COM3A1) | (1 << COM3A0) | (1 << COM3B1); // OC3A : set on compare match, OC3B : clear on compare match
  TCCR3B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR3
  icr = (F_CPU/1000000*period/2);
  int d = 1;
  while ((icr>0xFFFF)&&(d<5)) {
     d++;
     icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
  clockBits = d;
  uint16_t ocra = icr * (1.0-rapport);
  ICR3 = icr;
  OCR3A = ocra;
  OCR3B = ocra;
  TIMSK3 = 1 << TOIE3; // overflow interrupt enable
  sei();
  count = 0;
  max_count = nombre+1;
  TCNT3 = 0; // mise à zéro du compteur
  TCCR3B |= clockBits;
}
 
ISR(TIMER3_OVF_vect) { // Overflow interrupt
   count++;
    if (count==max_count) {
      TCCR3B &= ~0x7;
      TCNT3 = 0;
      TCCR3A = 0;
      PORTE &= ~((1 << PORTE3)|(1 << PORTE4));
   } 
}

void setup() {
  pinMode(ENABLE,OUTPUT);
  digitalWrite(ENABLE,HIGH);
  DDRE |= 1 << PORTE3; // data direction register
  DDRE |= 1 << PORTE4;
  PORTE &= ~((1 << PORTE3)|(1 << PORTE4)); // IN1 = 0, IN2 = 0 état de repos
  
}

void loop() {
  delay(1000);
  pulse_train(25,6,0.5);
}            
                     
#include "Arduino.h"

#define ENABLE 6
#define IN1 5 // OC3A (timer 3, Output compare A), PE3 (port E sortie 3)
#define IN2 2 // OC3B (timer 3, Output compare A), PE4 (port 3 sortie 4)
#define DETECT 48 // ICP5
 
char clockBits;
volatile uint32_t count;
int32_t max_count, count_start_detect;
volatile boolean start_detect;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
uint32_t icr;
volatile uint16_t delai;
uint8_t capture_clock = 2;


void pulse_train(uint32_t period, int32_t nombre, float rapport, float delai_detect) { 
  cli();
  TCCR3A = 0;
  TCCR3A |= (1 << COM3A1) | (1 << COM3A0) | (1 << COM3B1); // OC3A : set on compare match, OC3B : clear on compare match
  TCCR3B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR3
  icr = (F_CPU/1000000*period/2);
  int d = 1;
  while ((icr>0xFFFF)&&(d<5)) {
     d++;
     icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
  clockBits = d;
  count_start_detect = delai_detect/period;
  uint16_t ocra = icr * (1.0-rapport);
  ICR3 = icr;
  OCR3A = ocra;
  OCR3B = ocra;
  TIMSK3 = 1 << TOIE3; // overflow interrupt enable
  
  count = 0;
  max_count = nombre+1;
  TCNT3 = 0; // mise à zéro du compteur
  // Timer 5 en mode Input Capture
  TCCR5A = 0;
  TCNT5 = 0;
  TCCR5B = (1 << ICES5);
  TIMSK5 = (1 << ICIE5);
  
  // déclenchement des deux Timers
  TCCR3B |= clockBits;
  TCCR5B |= capture_clock; // 2 MHz 
  sei();
  start_detect = false;
  delai = 0;
}

ISR(TIMER3_OVF_vect) { // Overflow interrupt
   count++;
    if (count==max_count) {
      TCCR3A = 0;
      PORTE &= ~((1 << PORTE3)|(1 << PORTE4));
      
   } 
   if (count>=count_start_detect) start_detect = true;
   
}

ISR(TIMER5_CAPT_vect) {
   
   if (start_detect) {
      delai = ICR5; // input capture register
      TCCR5B = 0; // arrêt du Timer  
   }
}   

void setup() {
  Serial.begin(115200);
  pinMode(ENABLE,OUTPUT);
  digitalWrite(ENABLE,HIGH);
  pinMode(DETECT,INPUT);
  DDRE |= 1 << PORTE3; // data direction register
  DDRE |= 1 << PORTE4;
  PORTE &= ~((1 << PORTE3)|(1 << PORTE4)); // IN1 = 0, IN2 = 0 état de repos
  
}

void loop() {
  delay(500);
  pulse_train(25,15,0.5,1500);
  delay(500);
  float duree = delai*6.25e-5*diviseur[capture_clock]; // en ms
  Serial.println(duree,4);
  
}               
                